<?php
namespace Teknisa\EST\Modules\Ajuste\Services;

use Teknisa\EST\Services\ValidacoesEstoqueService;
use \Estoque\Entities\Lanctoestoq;
use \Estoque\Entities\Itlanctoest;
use \Estoque\Entities\Posestdia;
use \Estoque\Entities\Movilote;
use \Geral\Entities\Posestatu;
use \Geral\Entities\Paramfilial;
use Teknisa\Libs\Util\DateUtil;

class RecalculoService {

	protected $dataSourceName = "lanctoestoq_ajuste";
	
    protected $validacoesEstoqueService;
    protected $environment;

    public function __construct (ValidacoesEstoqueService $validacoesEstoqueService,$entityManager,$environment)
    {
        $this->validacoesEstoqueService = $validacoesEstoqueService;
        $this->entityManager = $entityManager;
        $this->environment = $environment;
    }
    
    public function deletePosestdia($dtposiestq,$cdfilial,$cdalmoxarife,$cdlocalestoq,$prodini,$prodfin) {
        try{
            $cdlocalestoq = $cdlocalestoq == ' '?null:$cdlocalestoq;
            $cdalmoxarife = $cdalmoxarife == ' '?null:$cdalmoxarife;
            Posestdia::getRepository()->deletaPosicaoEstoque($cdfilial, $cdalmoxarife, $cdlocalestoq, $prodini, $prodfin, $dtposiestq);
        }catch(\Exception $e){
            $this->entityManager->getConnection()->rollBack();
            throw $e;
        }
	}
	
	public function deletePosestatu($dtposiestq,$cdfilial,$cdalmoxarife,$cdlocalestoq,$prodini,$prodfin) {
        try{
            $dataMaisUm = DateUtil::getDataDeString($dtposiestq);
            $dataMaisUm = DateUtil::adicionaIntervalo($dataMaisUm)->format('d/m/Y');
            $result = Posestatu::getRepository()->consultaProdutosPosicaoEstoqueAtual($cdfilial, $cdalmoxarife, $cdlocalestoq, $prodini, $prodfin,$dtposiestq, $dataMaisUm);
            foreach ($result as $row) {
                Posestatu::deletaPosicaoEstoqueAtual($row['CDFILIAL'],$row['CDALMOXARIFE'],$row['CDLOCALESTOQ'],$row['NRLOTEESTQ'],$row['NRSUBLOTE'],$row['CDPRODUTO']);
            }
        }catch(\Exception $e){
            $this->entityManager->getConnection()->rollBack();
            throw $e;
        }
	}

    public function insereMovilote($dtposiestq,$cdfilial, $cdalmoxarife, $cdlocalestoq, $prodini, $prodfin, $nrmovilote){
        Movilote::insereMovimentacoesAPartirData($cdfilial, $cdalmoxarife, $cdlocalestoq, $prodini, $prodfin, $dtposiestq, $nrmovilote);
    }

    public function insertLogopera($cdfilial,$cdalmoxarife,$cdlocalestoq,$cdprodini,$cdprodfin,$dtinicial){
        $desc = 'Almox= '.  $cdalmoxarife.' Local='.  $cdlocalestoq.' ProdIni='. $cdprodini.' ProdFim='. $cdprodfin.' DataIncial='.$dtinicial;

        $sql = "INSERT INTO LOGOPERA
                   (NMTABELA,   DTOPERACAO, NMRESPOPER, IDTIPOOPER,     CDPESQITEM, DSOPERACAO)
                VALUES
                    ('RECALCULO',  SYSDATE, :P_USUARIO,'1', :P_CDFILIAL,  :P_DESC)";
        $params = array("P_USUARIO"   => $this->environment->getCdOperador(),
                       "P_CDFILIAL"    => $cdfilial,
                       "P_DESC" => $desc);
        $connection = $this->entityManager->getConnection();
        $query = $connection->prepare($sql);
        $query->execute($params);
    }
    
    public function atualizaEstoque($filial, $lancamento,$dtrecalculo){
        $nrorg = $this->environment->getNrOrgTrab();
        $dtprocessa = $this->validacoesEstoqueService->retornaDadosProcessFilial($filial,$nrorg)['DTPROCESSA'];
        $this->validacoesEstoqueService->executaProcedureAtualizaEstoque($filial, $filial, $lancamento, $dtrecalculo, $dtprocessa);
        return $this->verificaIrregularidades($filial, $lancamento,$nrorg);
    }
    
    public function verificaIrregularidades($cdfilial, $lancamento,$nrorg) {
        $nrmovilote = $cdfilial.$lancamento;

      
        $sql = "SELECT M.CDFILIAL AS CDFILIMOVI,   M.CDALMOXARIFE, M.CDLOCALESTOQ,
                       M.NRLOTEESTQ, M.NRSUBLOTE,    M.CDPRODUTO AS CDPRODMOVI,
                       P.CDARVPROD,  P.NMPRODUTO
                 FROM MOVILOTE M, PRODUTO P
                WHERE M.NRMOVILOTE = :P_NRMOVILOTE  AND
                      M.NRORG = :P_NRORG AND
                      M.CDPRODUTO  = P.CDPRODUTO AND
                      M.NRORG = P.NRORG";
        $params = array("P_NRMOVILOTE"   => $nrmovilote,
                        "P_NRORG"=>$nrorg);
        
        $connection = $this->entityManager->getConnection();
        $query = $connection->prepare($sql);
        $query->execute($params);
        
        $produtosIrregulares = $query->fetchAll();
         
        return $produtosIrregulares;
    }
    
    public function deleteMovilote($nrmovilote){

        $sql = "DELETE MOVILOTE
                 WHERE NRMOVILOTE = :P_NRMOVILOTE ";
        $params = array("P_NRMOVILOTE"   => $nrmovilote);
        $connection = $this->entityManager->getConnection();
        $query = $connection->prepare($sql);
        $query->execute($params);
    }
    
    
    // Formata a data de forma que possa ser inserida no banco
    public static function formataData($data) {
        if (!$data || !is_string($data))
            return $data;

        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);

        $data = DateUtil::getDataDeString($data);
        $data = DateUtil::truncateData($data);
        return $data;
    }
    

}